// html.cpp
void OpenHTTPHeader(connection * conn, char * type, int length, char * filename);
void OpenHtmlHeader(connection * conn);
void Style(connection * conn);
void RefreshHeader(connection * conn, char * url);
void CloseHeader(connection * conn);
void OpenPageBody(connection * conn);
void OpenHeaderBody(connection * conn);
void CloseBody(connection * conn);
void LinkTable(connection * conn);
void dot(connection * conn);
void nbsp(connection * conn);
void Img(connection * conn, int img, char * alt, int dx, int dy);
void ImgLink(connection * conn, char * link, int img, char * alt, int dx, int dy, int target);
void Link(connection * conn, char * link, char * text, int target);
void AboutSource(connection * conn);
void ImgLinkBar(connection * conn);
void LinkBar(connection * conn);
void FullSongInfo(connection * conn);
void SongInfo(connection * conn);
